@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 from argparse import ArgumentParser
 from pathlib import Path
-from shlex import quote
+from collections import defaultdict
 from subprocess import check_call, SubprocessError
 import sys
 from jinja2 import Environment, FileSystemLoader
@@ -37,13 +37,59 @@ def map_type(definition: dict, imported: bool = False):
 
     if type_name == 'integer':
         fmt = definition.get('format')
+        minimum = definition.get('minimum')
+        prefix = ''
+        if minimum == 0:
+            prefix = 'u'
         if fmt == 'int32':
-            return 'int32'
-        return 'int64'
+            return prefix + 'int32'
+        if fmt == 'int64':
+            return prefix + 'int64'
+        return prefix + 'int'
 
     return 'interface{}'
 
 
+def from_string(definition: dict, param: str) -> str:
+    type_name = map_type(definition)
+    if type_name == 'boolean':
+        return f'strconv.ParseBool({param})'
+    if type_name == 'float64':
+        return f'strconv.ParseFloat({param}, 64)'
+    if type_name == 'float32':
+        return f'strconv.ParseFloat({param}, 32)'
+    if type_name == 'int64' or type_name == 'int':
+        return f'strconv.ParseInt({param}, 10, 64)'
+    if type_name == 'int32':
+        return f'strconv.ParseInt({param}, 10, 32)'
+    if type_name == 'uint64' or type_name == 'uint':
+        return f'strconv.ParseUint({param}, 10, 64)'
+    if type_name == 'uint32':
+        return f'strconv.ParseUint({param}, 10, 32)'
+    return param
+
+
+def to_string(definition: dict, param: str) -> str:
+    type_name = map_type(definition)
+    if type_name == 'string':
+        return param
+    if type_name == 'boolean':
+        return f'strconv.FormatBool({param})'
+    if type_name == 'float64':
+        return f'strconv.FormatFloat({param}, \'f\', -1, 32)'
+    if type_name == 'float32':
+        return f'strconv.FormatFloat(float64({param}), \'f\', -1, 64)'
+    if type_name == 'int64':
+        return f'strconv.FormatInt({param}, 10)'
+    if type_name == 'int32' or type_name == 'int':
+        return f'strconv.FormatInt(int64({param}), 10)'
+    if type_name == 'uint64':
+        return f'strconv.FormatUint({param}, 10)'
+    if type_name == 'uint32' or type_name == 'uint':
+        return f'strconv.FormatUint(uint64({param}), 10)'
+    return f'fmt.Sprint({param})'
+
+
 def path(text: str) -> str:
     return re.sub(r'{(.*?)}', ':\\1', text)
 
@@ -91,6 +137,8 @@ def main():
     env.filters['label'] = lambda x: x[0].upper() + x[1:]
     env.filters['path'] = path
     env.filters['cast'] = cast
+    env.filters['from_string'] = from_string
+    env.filters['to_string'] = to_string
     env.filters['secured'] = lambda x: len(x.get('security', [])) > 0
     env.filters['sec_def'] = lambda x: swagger['securityDefinitions'][x]
     env.filters['has_payload'] = lambda x: any(param for param in x.get('parameters', []) if param['in'] == 'body')
@@ -107,6 +155,14 @@ def main():
     api_package = detect_package(args.output)
     package = api_package.split('/')[-1]
 
+    tags = defaultdict(set)
+    methods = {}
+    for http_methods in swagger.get('paths', {}).values():
+        for endpoint in http_methods.values():
+            methods[endpoint['operationId']] = endpoint
+            for tag in endpoint.get('tags', []):
+                tags[tag].add(endpoint['operationId'])
+
     base_file.write_text(env.get_template('base.jinja2').render(
         header=f"// Code generated by simple-swagger {' '.join(sys.argv[1:])} DO NOT EDIT.",
         swagger=swagger,
@@ -115,7 +171,10 @@ def main():
         api_package=api_package,
         enums=enums,
         has_security=len(
-            swagger.get('securityDefinitions', {})) > 0))
+            swagger.get('securityDefinitions', {})) > 0,
+        tags=tags,
+        methods=methods,
+    ))
 
     server_file.write_text(env.get_template('server.jinja2').render(
         header=f"// Code generated by simple-swagger {' '.join(sys.argv[1:])} DO NOT EDIT.",
